#include "mrk/parser_internal.h"

bool mrk_parser_done(mrk_parser *parser) {
  return mrk_lexer_done(parser->lexer) &&
         parser->next_token.type == mrk_token_type_none;
}

mrk_token mrk_parser_peek(mrk_parser *parser) { return parser->next_token; }

void mrk_parser_advance(mrk_parser *parser) {
  switch (mrk_lexer_next(&parser->next_token, parser->lexer)) {
  case mrk_lexer_err_ok:
  // TODO handle this better
  case mrk_lexer_err_unexpected_char:
    break;
  case mrk_lexer_err_done:
    parser->next_token.type = mrk_token_type_none;
    break;
  }
}

mrk_err mrk_parser_eat(mrk_token *out, mrk_parser *parser,
                       mrk_token_type type) {
  if (mrk_parser_done(parser) || mrk_parser_peek(parser).type != type) {
    parser->error.msg = "Unexpected token";
    parser->error.token = mrk_parser_peek(parser);

    return mrk_err_unexpected_token;
  }

  if (out != NULL) {
    *out = mrk_parser_peek(parser);
  }

  mrk_parser_advance(parser);

  return mrk_err_ok;
}

void mrk_parser_indent(mrk_parser *parser) {
  size_t seen = 0;

  while (!mrk_parser_done(parser) &&
         mrk_parser_peek(parser).type == mrk_token_type_indent) {
    mrk_parser_advance(parser);
    seen++;
  }

  parser->indent = seen;
}
